@@ -2,6 +2,7 @@
 #define CONTROLS_HPP_INCLUDED
 
 #include <vector>
+#include <cstddef>
 
 #include <SDL_keysym.h>
 
@@ -35,6 +36,7 @@ void read_local_controls();
 void ignore_current_keypresses();
 
 void get_control_status(int cycle, int player, bool* output);
+void set_delay(int delay);
 
 void read_control_packet(const char* buf, size_t len);
 void write_control_packet(std::vector<char>& v);
@@ -50,6 +52,8 @@ int cycles_behind();
 int their_highest_confirmed();
 int last_packet_size();
 
+void set_checksum(int cycle, int sum);
+
 void debug_dump_controls();
 
 }
