@@ -26,6 +26,7 @@
 #include "level_logic.hpp"
 #include "object_events.hpp"
 #include "playable_custom_object.hpp"
+#include "preferences.hpp"
 #include "raster.hpp"
 #include "string_utils.hpp"
 #include "surface_formula.hpp"
@@ -35,10 +36,19 @@
 #include "utils.hpp"
 #include "sound.hpp"
 
+namespace {
+const game_logic::formula_variable_storage_ptr& global_vars()
+{
+	static game_logic::formula_variable_storage_ptr obj(new game_logic::formula_variable_storage);
+	return obj;
+}
+}
+
 struct custom_object_text {
 	std::string text;
 	const_graphical_font_ptr font;
 	int size;
+	bool centered;
 	rect dimensions;
 	int alpha;
 };
@@ -59,7 +69,9 @@ custom_object::custom_object(wml::const_node_ptr node)
 	velocity_y_(wml::get_int(node, "velocity_y")),
 	accel_x_(wml::get_int(node, "accel_x")),
 	accel_y_(wml::get_int(node, "accel_y")),
+	gravity_shift_(wml::get_int(node, "gravity_shift", 0)),
 	rotate_(0), zorder_(wml::get_int(node, "zorder", type_->zorder())),
+	zsub_order_(wml::get_int(node, "zsub_order", 0)),
 	hitpoints_(wml::get_int(node, "hitpoints", type_->hitpoints())),
 	max_hitpoints_(wml::get_int(node, "max_hitpoints", type_->hitpoints())),
 	was_underwater_(false),
@@ -78,7 +90,11 @@ custom_object::custom_object(wml::const_node_ptr node)
 	vertex_shaders_(util::split(node->attr("vertex_shaders"))),
 	shader_(0),
 	always_active_(wml::get_bool(node, "always_active", false)),
-	last_cycle_active_(0)
+	last_cycle_active_(0),
+	parent_pivot_(node->attr("pivot")),
+	parent_prev_x_(0), parent_prev_y_(0), parent_prev_facing_(true),
+	min_difficulty_(wml::get_int(node, "min_difficulty", -1)),
+	max_difficulty_(wml::get_int(node, "max_difficulty", -1))
 {
 	if(node->has_attr("platform_area")) {
 		set_platform_area(rect(node->attr("platform_area")));
@@ -118,9 +134,14 @@ custom_object::custom_object(wml::const_node_ptr node)
 		const std::string& s = node->attr("rotation_schedule").str();
 
 		int nints = std::count(s.begin(), s.end(), ',')+1;
+		std::vector<int> rotation_ints(nints);
+		util::split_into_ints(s.c_str(), &rotation_ints[0], &nints);
 		position_schedule_->rotation.resize(nints);
-		util::split_into_ints(s.c_str(), &position_schedule_->rotation[0], &nints);
-		position_schedule_->rotation.resize(nints);
+		rotation_ints.resize(nints);
+		for(int n = 0; n != nints; ++n) {
+			position_schedule_->rotation[n] = decimal(rotation_ints[n]);
+		}
+
 	}
 
 	if(position_schedule_.get() != NULL && node->has_attr("schedule_speed")) {
@@ -222,7 +243,7 @@ custom_object::custom_object(wml::const_node_ptr node)
 
 	wml::const_node_ptr text_node = node->get_child("text");
 	if(text_node) {
-		set_text(text_node->attr("text"), text_node->attr("font"), wml::get_int(text_node, "size", 2));
+		set_text(text_node->attr("text"), text_node->attr("font"), wml::get_int(text_node, "size", 2), wml::get_bool(text_node, "centered", false));
 	}
 
 	if(node->has_attr("particles")) {
@@ -238,6 +259,9 @@ custom_object::custom_object(wml::const_node_ptr node)
 			lights_.push_back(new_light);
 		}
 	}
+	
+	//fprintf(stderr, "object address= %p, ", this);
+	//fprintf(stderr, "zsub_order=%d,", zsub_order_);
 }
 
 custom_object::custom_object(const std::string& type, int x, int y, bool face_right)
@@ -249,8 +273,9 @@ custom_object::custom_object(const std::string& type, int x, int y, bool face_ri
     frame_name_("normal"),
 	time_in_frame_(0), time_in_frame_delta_(1),
 	velocity_x_(0), velocity_y_(0),
-	accel_x_(0), accel_y_(0),
+	accel_x_(0), accel_y_(0), gravity_shift_(0),
 	rotate_(0), zorder_(type_->zorder()),
+	zsub_order_(0),
 	hitpoints_(type_->hitpoints()),
 	max_hitpoints_(type_->hitpoints()),
 	was_underwater_(false),
@@ -265,7 +290,9 @@ custom_object::custom_object(const std::string& type, int x, int y, bool face_ri
 	loaded_(false), fall_through_platforms_(0),
 	shader_(0),
 	always_active_(false),
-	last_cycle_active_(0)
+	last_cycle_active_(0),
+	parent_prev_x_(0), parent_prev_y_(0), parent_prev_facing_(true),
+	min_difficulty_(-1), max_difficulty_(-1)
 {
 	set_solid_dimensions(type_->solid_dimensions(),
 	                     type_->weak_solid_dimensions());
@@ -298,8 +325,10 @@ custom_object::custom_object(const custom_object& o) :
 	time_in_frame_delta_(o.time_in_frame_delta_),
 	velocity_x_(o.velocity_x_), velocity_y_(o.velocity_y_),
 	accel_x_(o.accel_x_), accel_y_(o.accel_y_),
+	gravity_shift_(o.gravity_shift_),
 	rotate_(o.rotate_),
 	zorder_(o.zorder_),
+	zsub_order_(o.zsub_order_),
 	hitpoints_(o.hitpoints_),
 	max_hitpoints_(o.max_hitpoints_),
 	was_underwater_(o.was_underwater_),
@@ -332,7 +361,14 @@ custom_object::custom_object(const custom_object& o) :
 	vertex_shaders_(o.vertex_shaders_),
 	shader_(o.shader_),
 	always_active_(o.always_active_),
-	last_cycle_active_(0)
+	last_cycle_active_(0),
+	parent_(o.parent_),
+	parent_pivot_(o.parent_pivot_),
+	parent_prev_x_(o.parent_prev_x_),
+	parent_prev_y_(o.parent_prev_y_),
+	parent_prev_facing_(o.parent_prev_facing_),
+	min_difficulty_(o.min_difficulty_),
+	max_difficulty_(o.max_difficulty_)
 {
 }
 
@@ -368,7 +404,12 @@ wml::node_ptr custom_object::write() const
 		}
 
 		if(position_schedule_->rotation.empty() == false) {
-			res->set_attr("rotation_schedule", util::join_ints(&position_schedule_->rotation[0], position_schedule_->rotation.size()));
+			std::vector<int> v(position_schedule_->rotation.size());
+			for(int n = 0; n != v.size(); ++n) {
+				v[n] = position_schedule_->rotation[n].value();
+			}
+
+			res->set_attr("rotation_schedule", util::join_ints(&v[0], v.size()));
 		}
 	}
 
@@ -376,6 +417,10 @@ wml::node_ptr custom_object::write() const
 		std::string s;
 
 		foreach(const entity_ptr& e, attached_objects()) {
+			if(s.empty() == false) {
+				s += ",";
+			}
+
 			char buf[256];
 			sprintf(buf, "%p", e.get());
 			s += buf;
@@ -410,6 +455,10 @@ wml::node_ptr custom_object::write() const
 	res->set_attr("y", formatter() << y());
 	res->set_attr("velocity_x", formatter() << velocity_x_);
 	res->set_attr("velocity_y", formatter() << velocity_y_);
+	
+	if(platform_motion_x()) {
+		res->set_attr("platform_motion_x", formatter() << platform_motion_x());
+	}
 
 	if(solid_dimensions() != type_->solid_dimensions() ||
 	   weak_solid_dimensions() != type_->weak_solid_dimensions()) {
@@ -484,6 +533,10 @@ wml::node_ptr custom_object::write() const
 		res->set_attr("zorder", formatter() << zorder_);
 	}
 
+	if(zsub_order_ != 0) {
+		res->set_attr("zsub_order", formatter() << zsub_order_);
+	}
+	
 	res->set_attr("face_right", face_right() ? "yes" : "no");
 	if(upside_down()) {
 		res->set_attr("upside_down", "yes");
@@ -527,12 +580,6 @@ wml::node_ptr custom_object::write() const
 		res->add_child(wml::deep_copy(custom_type_));
 	}
 
-#ifndef NO_EDITOR
-	if(editor_info()) {
-		res->add_child(editor_info()->write());
-	}
-#endif
-
 	if(text_) {
 		wml::node_ptr node(new wml::node("text"));
 		node->set_attr("text", text_->text);
@@ -575,6 +622,24 @@ wml::node_ptr custom_object::write() const
 	foreach(const light_ptr& p, lights_) {
 		res->add_child(p->write());
 	}
+
+	if(parent_.get() != NULL) {
+		char buf[256];
+		sprintf(buf, "%p", parent_.get());
+		res->set_attr("parent", buf);
+	}
+
+	if(parent_pivot_.empty() == false) {
+		res->set_attr("pivot", parent_pivot_);
+	}
+
+	if(min_difficulty_ != -1) {
+		res->set_attr("min_difficulty", formatter() << min_difficulty_);
+	}
+
+	if(max_difficulty_ != -1) {
+		res->set_attr("max_difficulty", formatter() << max_difficulty_);
+	}
 	
 	return res;
 }
@@ -625,9 +690,9 @@ void custom_object::draw() const
 	const int draw_y = y();
 
 	if(!draw_area_.get()) {
-		frame_->draw(draw_x, draw_y, face_right(), upside_down(), time_in_frame_, rotate_);
+		frame_->draw(draw_x-draw_x%2, draw_y-draw_y%2, face_right(), upside_down(), time_in_frame_, rotate_.as_float());
 	} else {
-		frame_->draw(draw_x, draw_y, *draw_area_, face_right(), upside_down(), time_in_frame_, rotate_);
+		frame_->draw(draw_x-draw_x%2, draw_y-draw_y%2, *draw_area_, face_right(), upside_down(), time_in_frame_, rotate_.as_float());
 	}
 
 	if(blur_) {
@@ -641,7 +706,7 @@ void custom_object::draw() const
 			while(!transform.fits_in_color()) {
 				transform = transform - transform.to_color();
 				transform.to_color().set_as_current_color();
-				frame_->draw(draw_x, draw_y, face_right(), upside_down(), time_in_frame_, rotate_);
+				frame_->draw(draw_x-draw_x%2, draw_y-draw_y%2, face_right(), upside_down(), time_in_frame_, rotate_.as_float());
 			}
 
 			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -662,7 +727,10 @@ void custom_object::draw() const
 
 	if(text_ && text_->font && text_->alpha) {
 		glColor4ub(255, 255, 255, text_->alpha);
-		text_->font->draw(draw_x, draw_y, text_->text, text_->size);
+		if (text_->centered)
+			text_->font->draw(draw_x-text_->dimensions.w()/2, draw_y, text_->text, text_->size);
+		else
+			text_->font->draw(draw_x, draw_y, text_->text, text_->size);
 		glColor4ub(255, 255, 255, 255);
 	}
 
@@ -692,6 +760,27 @@ void custom_object::process(level& lvl)
 		return;
 	}
 
+	ASSERT_LOG(!entity_collides(level::current(), *this, MOVE_NONE), "ENTITY " << type_->id() << " COLLIDES AT START OF PROCESS");
+
+	if(parent_.get() != NULL) {
+		const point pos = parent_position();
+		const int move_x = pos.x - parent_prev_x_;
+		const int move_y = pos.y - parent_prev_y_;
+
+
+		const bool parent_facing = parent_->face_right();
+		if(parent_facing != parent_prev_facing_) {
+			const int relative_x = x() + current_frame().width()/2 - parent_position().x;
+			move_centipixels(-relative_x*200, 0);
+		} else {
+			move_centipixels(move_x*100, move_y*100);
+		}
+
+		parent_prev_x_ = pos.x;
+		parent_prev_y_ = pos.y;
+		parent_prev_facing_ = parent_facing;
+	}
+
 	if(last_cycle_active_ < lvl.cycle() - 5) {
 		handle_event(OBJECT_EVENT_BECOME_ACTIVE);
 	}
@@ -704,6 +793,12 @@ void custom_object::process(level& lvl)
 //	assert(!entity_collides_with_level(lvl, *this, MOVE_NONE));
 //	assert(!entity_collides(lvl, *this, MOVE_NONE));
 
+	//this is a flag which tracks whether we've fired a collide_feet
+	//event. If we don't fire a collide_feet event through normal collision
+	//detection, but we change the object we're standing on, we should
+	//still fire a collide_feet event.
+	bool fired_collide_feet = false;
+
 	collision_info stand_info;
 	const bool started_standing = is_standing(lvl, &stand_info);
 	if(!started_standing && standing_on_) {
@@ -755,14 +850,14 @@ void custom_object::process(level& lvl)
 		int xpos = INT_MIN, ypos = INT_MIN;
 		if(position_schedule_->x_pos.empty() == false) {
 			xpos = position_schedule_->x_pos[pos%position_schedule_->x_pos.size()];
-			if(next_fraction) {
+			if(next_fraction && pos+1 != position_schedule_->x_pos.size()) {
 				xpos = (xpos*this_fraction + next_fraction*position_schedule_->x_pos[(pos+1)%position_schedule_->x_pos.size()])/position_schedule_->speed;
 			}
 		}
 
 		if(position_schedule_->y_pos.empty() == false) {
 			ypos = position_schedule_->y_pos[pos%position_schedule_->y_pos.size()];
-			if(next_fraction) {
+			if(next_fraction && pos+1 != position_schedule_->y_pos.size()) {
 				ypos = (ypos*this_fraction + next_fraction*position_schedule_->y_pos[(pos+1)%position_schedule_->y_pos.size()])/position_schedule_->speed;
 			}
 		}
@@ -776,9 +871,9 @@ void custom_object::process(level& lvl)
 		}
 
 		if(position_schedule_->rotation.empty() == false) {
-			rotate_ = position_schedule_->rotation[pos%position_schedule_->rotation.size()];
+			rotate_ = decimal(position_schedule_->rotation[pos%position_schedule_->rotation.size()]);
 			if(next_fraction) {
-				rotate_ = (rotate_*this_fraction + next_fraction*position_schedule_->rotation[(pos+1)%position_schedule_->rotation.size()])/position_schedule_->speed;
+				rotate_ = decimal((rotate_*this_fraction + next_fraction*position_schedule_->rotation[(pos+1)%position_schedule_->rotation.size()])/position_schedule_->speed);
 			}
 		}
 	}
@@ -826,7 +921,7 @@ void custom_object::process(level& lvl)
 	velocity_x_ += (accel_x_ * (stand_info.traction ? traction_from_surface : (is_underwater?type_->traction_in_water() :type_->traction_in_air())) * (face_right() ? 1 : -1))/1000;
 	if(!standing_on_ && !started_standing || accel_y_ < 0) {
 		//do not accelerate downwards if standing on something.
-		velocity_y_ += accel_y_ * (is_underwater ? type_->traction_in_water() : 1000)/1000;
+		velocity_y_ += accel_y_ * (gravity_shift_ + (is_underwater ? type_->traction_in_water() : 1000))/1000;
 	}
 
 	if(type_->friction()) {
@@ -861,7 +956,7 @@ void custom_object::process(level& lvl)
 	}
 
 	if(standing_on_) {
-		effective_velocity_x += (standing_on_->feet_x() - standing_on_prev_x_)*100;
+		effective_velocity_x += (standing_on_->feet_x() - standing_on_prev_x_)*100 + standing_on_->platform_motion_x();
 		effective_velocity_y += (standing_on_->feet_y() - standing_on_prev_y_)*100;
 	}
 
@@ -993,6 +1088,7 @@ void custom_object::process(level& lvl)
 			}
 
 			handle_event(effective_velocity_y < 0 ? OBJECT_EVENT_COLLIDE_HEAD : OBJECT_EVENT_COLLIDE_FEET, callable);
+			fired_collide_feet = true;
 		}
 
 		if(collide_info.damage || jump_on_info.damage) {
@@ -1023,7 +1119,7 @@ void custom_object::process(level& lvl)
 				handle_event(OBJECT_EVENT_COLLIDE_LEVEL);
 			}
 
-			const bool previous_standing = is_standing(lvl);
+			const STANDING_STATUS previous_standing = is_standing(lvl);
 
 			const int dir = effective_velocity_x > 0 ? 1 : -1;
 			const int original_centi_y = centi_y();
@@ -1040,8 +1136,8 @@ void custom_object::process(level& lvl)
 			//move the character up or down as appropriate to try to keep
 			//them standing.
 
-			const bool standing = is_standing(lvl);
-			if(previous_standing && !standing) {
+			const STANDING_STATUS standing = is_standing(lvl);
+			if(previous_standing && standing < previous_standing) {
 
 				//we were standing, but we're not now. We want to look for
 				//slopes that will enable us to still be standing. We see
@@ -1059,7 +1155,7 @@ void custom_object::process(level& lvl)
 							break;
 						}
 
-						if(is_standing(lvl)) {
+						if(is_standing(lvl) >= previous_standing) {
 							resolved = true;
 							break;
 						}
@@ -1169,7 +1265,8 @@ void custom_object::process(level& lvl)
 			}
 		}
 
-		handle_event(collide ? OBJECT_EVENT_COLLIDE : OBJECT_EVENT_COLLIDE_FEET, callable);
+		handle_event(collide ? OBJECT_EVENT_COLLIDE_SIDE : OBJECT_EVENT_COLLIDE_FEET, callable);
+		fired_collide_feet = true;
 		if(collide_info.damage) {
 			game_logic::map_formula_callable* callable = new game_logic::map_formula_callable;
 			callable->add("surface_damage", variant(collide_info.damage));
@@ -1191,14 +1288,17 @@ void custom_object::process(level& lvl)
 	if(standing_on_ && standing_on_ != stand_info.collide_with) {
 		//we were previously standing on an object and we're not anymore.
 		//add the object we were standing on's velocity to ours
-		velocity_x_ += standing_on_->last_move_x()*100;
+		velocity_x_ += standing_on_->last_move_x()*100 + standing_on_->platform_motion_x();
 		velocity_y_ += standing_on_->last_move_y()*100;
 	}
 
 	if(stand_info.collide_with && standing_on_ != stand_info.collide_with) {
+		if(!fired_collide_feet) {
+		}
+
 		//we are standing on a new object. Adjust our velocity relative to
 		//the object we're standing on
-		velocity_x_ -= stand_info.collide_with->last_move_x()*100;
+		velocity_x_ -= stand_info.collide_with->last_move_x()*100 + stand_info.collide_with->platform_motion_x();
 		velocity_y_ = 0;
 
 		game_logic::map_formula_callable* callable(new game_logic::map_formula_callable(this));
@@ -1231,7 +1331,7 @@ void custom_object::process(level& lvl)
 	}
 
 	for(std::map<std::string, particle_system_ptr>::iterator i = particle_systems_.begin(); i != particle_systems_.end(); ) {
-		i->second->process(lvl, *this);
+		i->second->process(*this);
 		if(i->second->is_destroyed()) {
 			particle_systems_.erase(i++);
 		} else {
@@ -1242,7 +1342,7 @@ void custom_object::process(level& lvl)
 	set_driver_position();
 
 	if(blur_) {
-		blur_->next_frame(start_x, start_y, x(), y(), frame_, time_in_frame_, face_right(), upside_down(), rotate_);
+		blur_->next_frame(start_x, start_y, x(), y(), frame_, time_in_frame_, face_right(), upside_down(), rotate_.value());
 		if(blur_->destroyed()) {
 			blur_.reset();
 		}
@@ -1274,6 +1374,11 @@ int custom_object::zorder() const
 	return zorder_;
 }
 
+int custom_object::zsub_order() const
+{
+	return zsub_order_;
+}
+
 int custom_object::velocity_x() const
 {
 	return velocity_x_;
@@ -1422,11 +1527,32 @@ void custom_object::control(const level& lvl)
 {
 }
 
-bool custom_object::is_standing(const level& lvl, collision_info* info) const
+custom_object::STANDING_STATUS custom_object::is_standing(const level& lvl, collision_info* info) const
 {
-	const bool result = has_feet() &&
-	       point_standable(lvl, *this, feet_x(), feet_y(), info, fall_through_platforms_ ? SOLID_ONLY : SOLID_AND_PLATFORMS);
-	return result;
+	if(!has_feet()) {
+		return NOT_STANDING;
+	}
+
+	const int width = type_->feet_width();
+
+	if(width >= 1) {
+		const int facing = face_right() ? 1 : -1;
+		if(point_standable(lvl, *this, feet_x() + width*facing, feet_y(), info, fall_through_platforms_ ? SOLID_ONLY : SOLID_AND_PLATFORMS)) {
+			return STANDING_FRONT_FOOT;
+		}
+
+		if(point_standable(lvl, *this, feet_x() - width*facing, feet_y(), info, fall_through_platforms_ ? SOLID_ONLY : SOLID_AND_PLATFORMS)) {
+			return STANDING_BACK_FOOT;
+		}
+
+		return NOT_STANDING;
+	}
+
+	if(point_standable(lvl, *this, feet_x(), feet_y(), info, fall_through_platforms_ ? SOLID_ONLY : SOLID_AND_PLATFORMS)) {
+		return STANDING_FRONT_FOOT;
+	} else {
+		return NOT_STANDING;
+	}
 }
 
 namespace {
@@ -1471,6 +1597,10 @@ variant custom_object::get_value_by_slot(int slot) const
 	case CUSTOM_OBJECT_Y:                 return variant(y());
 	case CUSTOM_OBJECT_Z:
 	case CUSTOM_OBJECT_ZORDER:            return variant(zorder_);
+	case CUSTOM_OBJECT_RELATIVE_X:        return variant(x() - parent_position().x);
+	case CUSTOM_OBJECT_RELATIVE_Y:        return variant(y() - parent_position().y);
+	case CUSTOM_OBJECT_PARENT:            return variant(parent_.get());
+	case CUSTOM_OBJECT_PIVOT:             return variant(parent_pivot_);
 	case CUSTOM_OBJECT_PREVIOUS_Y:        return variant(previous_y_);
 	case CUSTOM_OBJECT_X1:                return variant(solid_rect().x());
 	case CUSTOM_OBJECT_X2:                return variant(solid_rect().x2());
@@ -1495,6 +1625,10 @@ variant custom_object::get_value_by_slot(int slot) const
 	case CUSTOM_OBJECT_VELOCITY_Y:        return variant(velocity_y_);
 	case CUSTOM_OBJECT_ACCEL_X:           return variant(accel_x_);
 	case CUSTOM_OBJECT_ACCEL_Y:           return variant(accel_y_);
+	case CUSTOM_OBJECT_GRAVITY_SHIFT:     return variant(gravity_shift_);
+	case CUSTOM_OBJECT_PLATFORM_MOTION_X: return variant(platform_motion_x());
+	case CUSTOM_OBJECT_REGISTRY:          return variant(preferences::registry());
+	case CUSTOM_OBJECT_GLOBALS:           return variant(global_vars().get());
 	case CUSTOM_OBJECT_VARS:              return variant(vars_.get());
 	case CUSTOM_OBJECT_TMP:               return variant(tmp_vars_.get());
 	case CUSTOM_OBJECT_GROUP:             return variant(group());
@@ -1506,7 +1640,8 @@ variant custom_object::get_value_by_slot(int slot) const
 	case CUSTOM_OBJECT_BLUE:              return variant(draw_color().b());
 	case CUSTOM_OBJECT_ALPHA:             return variant(draw_color().a());
 	case CUSTOM_OBJECT_TEXT_ALPHA:        return variant(text_ ? text_->alpha : 255);
-	case CUSTOM_OBJECT_DAMAGE:            return variant(current_frame().damage()); case CUSTOM_OBJECT_HIT_BY:            return variant(last_hit_by_.get());
+	case CUSTOM_OBJECT_DAMAGE:            return variant(current_frame().damage());
+	case CUSTOM_OBJECT_HIT_BY:            return variant(last_hit_by_.get());
 	case CUSTOM_OBJECT_DISTORTION:        return variant(distortion_.get());
 	case CUSTOM_OBJECT_IS_STANDING:       return variant(standing_on_.get() || is_standing(level::current()));
 	case CUSTOM_OBJECT_NEAR_CLIFF_EDGE:   return variant(is_standing(level::current()) && cliff_edge_within(level::current(), feet_x(), feet_y(), face_dir()*15));
@@ -1628,6 +1763,13 @@ variant custom_object::get_value_by_slot(int slot) const
 		return variant(&result);
 	}
 
+	case CUSTOM_OBJECT_SOLID_DIMENSIONS_IN: {
+		std::vector<variant> v;
+		v.push_back(variant(solid_dimensions()));
+		v.push_back(variant(weak_solid_dimensions()));
+		return variant(&v);
+	}
+
 	case CUSTOM_OBJECT_ALWAYS_ACTIVE: return variant(always_active_);
 	case CUSTOM_OBJECT_TAGS: return variant(tags_.get());
 	case CUSTOM_OBJECT_HAS_FEET: return variant(has_feet_);
@@ -1678,6 +1820,10 @@ variant custom_object::get_value(const std::string& key) const
 	}
 
 	if(backup_callable_stack_.empty() == false && backup_callable_stack_.top()) {
+		if(key == "arg") {
+			return variant(backup_callable_stack_.top());
+		}
+
 		return backup_callable_stack_.top()->query_value(key);
 	}
 
@@ -1752,7 +1898,7 @@ void custom_object::set_value(const std::string& key, const variant& value)
 	} else if(key == "accel_y") {
 		accel_y_ = value.as_int();
 	} else if(key == "rotate") {
-		rotate_ = value.as_int();
+		rotate_ = value.as_decimal();
 	} else if(key == "red") {
 		make_draw_color();
 		draw_color_->buf()[0] = truncate_to_char(value.as_int());
@@ -1901,11 +2047,16 @@ void custom_object::set_value(const std::string& key, const variant& value)
 			game_logic::formula_variable_storage_ptr old_vars = vars_, old_tmp_vars_ = tmp_vars_;
 
 			type_ = p;
+			has_feet_ = type_->has_feet();
 			vars_.reset(new game_logic::formula_variable_storage(type_->variables())),
 			tmp_vars_.reset(new game_logic::formula_variable_storage(type_->tmp_variables())),
 
 			vars_->add(*old_vars);
 			tmp_vars_->add(*old_tmp_vars_);
+
+			//set the animation to the default animation for the new type.
+			set_frame(type_->default_frame().id());
+			//std::cerr << "SET TYPE WHEN CHANGING TO '" << type_->id() << "'\n";
 		}
 	} else {
 		vars_->add(key, value);
@@ -1921,11 +2072,16 @@ void custom_object::set_value_by_slot(int slot, const variant& value)
 			game_logic::formula_variable_storage_ptr old_vars = vars_, old_tmp_vars_ = tmp_vars_;
 
 			type_ = p;
+			has_feet_ = type_->has_feet();
 			vars_.reset(new game_logic::formula_variable_storage(type_->variables())),
 			tmp_vars_.reset(new game_logic::formula_variable_storage(type_->tmp_variables())),
 
 			vars_->add(*old_vars);
 			tmp_vars_->add(*old_tmp_vars_);
+
+			//set the animation to the default animation for the new type.
+			set_frame(type_->default_frame().id());
+			//std::cerr << "SET TYPE WHEN CHANGING TO '" << type_->id() << "'\n";
 		}
 	}
 		break;
@@ -1964,20 +2120,55 @@ void custom_object::set_value_by_slot(int slot, const variant& value)
 		zorder_ = value.as_int();
 		break;
 	
+	case CUSTOM_OBJECT_RELATIVE_X: {
+		const point p = parent_position();
+		set_value_by_slot(CUSTOM_OBJECT_X, variant(p.x + value.as_int()));
+		break;
+	}
+
+	case CUSTOM_OBJECT_RELATIVE_Y: {
+		const point p = parent_position();
+		set_value_by_slot(CUSTOM_OBJECT_Y, variant(p.y + value.as_int()));
+		break;
+	}
+
+	case CUSTOM_OBJECT_PARENT: {
+		entity_ptr e(value.try_convert<entity>());
+		set_parent(e, parent_pivot_);
+		break;
+	}
+
+	case CUSTOM_OBJECT_PIVOT: {
+		set_parent(parent_, value.as_string());
+		break;
+	}
+	
 	case CUSTOM_OBJECT_MIDPOINT_X: {
+		const int start_x = centi_x();
 		const int current_x = x() + current_frame().width()/2;
 		const int xdiff = current_x - x();
 		set_pos(value.as_int() - xdiff, y());
+		if(entity_collides(level::current(), *this, MOVE_NONE) && entity_in_current_level(this)) {
+			set_centi_x(start_x);
+		}
 		break;
 	}
 
 	case CUSTOM_OBJECT_MIDPOINT_Y: {
+		const int start_y = centi_y();
 		const int current_y = y() + current_frame().height()/2;
 		const int ydiff = current_y - y();
 		set_pos(x(), value.as_int() - ydiff);
+		if(entity_collides(level::current(), *this, MOVE_NONE) && entity_in_current_level(this)) {
+			set_centi_y(start_y);
+		}
 		break;
 	}
 
+	case CUSTOM_OBJECT_CYCLE:
+		cycle_ = value.as_int();
+		break;
+
 	case CUSTOM_OBJECT_FACING:
 		set_face_right(value.as_int() > 0);
 		break;
@@ -2017,10 +2208,18 @@ void custom_object::set_value_by_slot(int slot, const variant& value)
 		accel_y_ = value.as_int();
 		break;
 
-	case CUSTOM_OBJECT_ROTATE:
-		rotate_ = value.as_int();
+	case CUSTOM_OBJECT_GRAVITY_SHIFT:
+		gravity_shift_ = value.as_int();
 		break;
-	
+
+	case CUSTOM_OBJECT_PLATFORM_MOTION_X:
+		set_platform_motion_x(value.as_int());
+		break;
+
+	case CUSTOM_OBJECT_ROTATE:
+		rotate_ = value.as_decimal();
+		break;
+
 	case CUSTOM_OBJECT_RED:
 		make_draw_color();
 		draw_color_->buf()[0] = truncate_to_char(value.as_int());
@@ -2043,7 +2242,7 @@ void custom_object::set_value_by_slot(int slot, const variant& value)
 
 	case CUSTOM_OBJECT_TEXT_ALPHA:
 		if(!text_) {
-			set_text("", "default", 10);
+			set_text("", "default", 10, false);
 		}
 
 		text_->alpha = value.as_int();
@@ -2265,7 +2464,7 @@ void custom_object::set_value_by_slot(int slot, const variant& value)
 
 		position_schedule_->rotation.clear();
 		for(int n = 0; n != value.num_elements(); ++n) {
-			position_schedule_->rotation.push_back(value[n].as_int());
+			position_schedule_->rotation.push_back(value[n].as_decimal());
 		}
 		break;
 	}
@@ -2381,8 +2580,20 @@ void custom_object::die()
 	handle_event(OBJECT_EVENT_DIE);
 }
 
+void custom_object::die_with_no_event()
+{
+	hitpoints_ = 0;
+}
+
+
 bool custom_object::is_active(const rect& screen_area) const
 {
+	if(controls::num_players() > 1) {
+		//in multiplayer, make all objects always active
+		//TODO: review this behavior
+		return true;
+	}
+
 	if(always_active()) {
 		return true;
 	}
@@ -2412,10 +2623,11 @@ bool custom_object::is_active(const rect& screen_area) const
 	}
 
 	const rect& area = frame_rect();
-	if(area.x() < screen_area.x2() + 100 &&
-	   area.x2() > screen_area.x() - 100 &&
-	   area.y() < screen_area.y2() + 100 &&
-	   area.y2() > screen_area.y() - 100) {
+	const int border = type_->activation_border();
+	if(area.x() < screen_area.x2() + border &&
+	   area.x2() > screen_area.x() - border &&
+	   area.y() < screen_area.y2() + border &&
+	   area.y2() > screen_area.y() - border) {
 		return true;
 	}
 
@@ -2707,12 +2919,13 @@ void custom_object::remove_particle_system(const std::string& key)
 	particle_systems_.erase(key);
 }
 
-void custom_object::set_text(const std::string& text, const std::string& font, int size)
+void custom_object::set_text(const std::string& text, const std::string& font, int size, bool centered)
 {
 	text_.reset(new custom_object_text);
 	text_->text = text;
 	text_->font = graphical_font::get(font);
 	text_->size = size;
+	text_->centered = centered;
 	text_->alpha = 255;
 	ASSERT_LOG(text_->font, "UNKNOWN FONT: " << font);
 	text_->dimensions = text_->font->dimensions(text_->text);
@@ -2818,6 +3031,24 @@ void custom_object::set_platform_area(const rect& area)
 void custom_object::shift_position(int x, int y)
 {
 	entity::shift_position(x, y);
+	if(standing_on_prev_x_ != INT_MIN) {
+		standing_on_prev_x_ += x;
+	}
+
+	if(standing_on_prev_y_ != INT_MIN) {
+		standing_on_prev_y_ += y;
+	}
+
+	if(position_schedule_.get() != NULL) {
+		foreach(int& xpos, position_schedule_->x_pos) {
+			xpos += x;
+		}
+
+		foreach(int& ypos, position_schedule_->y_pos) {
+			ypos += y;
+		}
+	}
+
 	if(activation_area_.get() != NULL) {
 		activation_area_.reset(new rect(activation_area_->x() + x,
 		                                activation_area_->y() + y,
@@ -2826,6 +3057,35 @@ void custom_object::shift_position(int x, int y)
 	}
 }
 
+bool custom_object::appears_at_difficulty(int difficulty) const
+{
+	return (min_difficulty_ == -1 || difficulty >= min_difficulty_) &&
+	       (max_difficulty_ == -1 || difficulty <= max_difficulty_);
+}
+
+void custom_object::set_parent(entity_ptr e, const std::string& pivot_point)
+{
+	parent_ = e;
+	parent_pivot_ = pivot_point;
+
+	const point pos = parent_position();
+	parent_prev_x_ = pos.x;
+	parent_prev_y_ = pos.y;
+
+	if(parent_.get() != NULL) {
+		parent_prev_facing_ = parent_->face_right();
+	}
+}
+
+point custom_object::parent_position() const
+{
+	if(parent_.get() == NULL) {
+		return point(0,0);
+	}
+
+	return parent_->pivot(parent_pivot_);
+}
+
 BENCHMARK(custom_object_spike) {
 	static level* lvl = NULL;
 	if(!lvl) {	
