@@ -4,6 +4,7 @@
 #include <iostream>
 #include <math.h>
 
+#include "IMG_savepng.h"
 #include "asserts.hpp"
 #include "collision_utils.hpp"
 #include "controls.hpp"
@@ -22,6 +23,7 @@
 #include "multiplayer.hpp"
 #include "object_events.hpp"
 #include "player_info.hpp"
+#include "playable_custom_object.hpp"
 #include "preferences.hpp"
 #include "preprocessor.hpp"
 #include "random.hpp"
@@ -29,6 +31,7 @@
 #include "stats.hpp"
 #include "string_utils.hpp"
 #include "surface_palette.hpp"
+#include "texture_frame_buffer.hpp"
 #include "thread.hpp"
 #include "tile_map.hpp"
 #include "unit_test.hpp"
@@ -82,23 +85,54 @@ void level::set_as_current_level()
 {
 	current_level = this;
 	frame::set_color_palette(palettes_used_);
+
+#if !TARGET_OS_IPHONE
+	static const int starting_x_resolution = preferences::actual_screen_width();
+	static const int starting_y_resolution = preferences::actual_screen_height();
+	static const int starting_virtual_x_resolution = preferences::virtual_screen_width();
+	static const int starting_virtual_y_resolution = preferences::virtual_screen_height();
+
+	if(!editor_ && starting_x_resolution == starting_virtual_x_resolution) {
+		if(!x_resolution_) {
+			x_resolution_ = starting_x_resolution;
+		}
+
+		if(!y_resolution_) {
+			y_resolution_ = starting_y_resolution;
+		}
+
+
+		if(x_resolution_ != preferences::actual_screen_width() || y_resolution_ != preferences::actual_screen_height()) {
+			const bool result = graphics::set_video_mode(x_resolution_, y_resolution_);
+			if(result) {
+				preferences::set_actual_screen_width(x_resolution_);
+				preferences::set_actual_screen_height(y_resolution_);
+				preferences::set_virtual_screen_width(x_resolution_);
+				preferences::set_virtual_screen_height(y_resolution_);
+			}
+		}
+	}
+	
+#endif
 }
 
 namespace {
 graphics::color_transform default_dark_color() {
-	return graphics::color_transform(8, 22, 35, 255);
+	return graphics::color_transform(0, 0, 0, 0);
 }
 }
 
 level::level(const std::string& level_cfg)
-	: id_(level_cfg), highlight_layer_(INT_MIN),
+	: id_(level_cfg),
+	  x_resolution_(0), y_resolution_(0),
+	  highlight_layer_(INT_MIN),
 	  num_compiled_tiles_(0),
 	  entered_portal_active_(false), save_point_x_(-1), save_point_y_(-1),
 	  editor_(false), show_foreground_(true), show_background_(true), dark_(false), dark_color_(graphics::color_transform(0, 0, 0, 255)), air_resistance_(0), water_resistance_(7), end_game_(false),
       editor_tile_updates_frozen_(0), zoom_level_(1),
 	  palettes_used_(0),
 	  background_palette_(-1),
-	  segment_width_(0)
+	  segment_width_(0), segment_height_(0)
 {
 	std::cerr << "in level constructor...\n";
 	const int start_time = SDL_GetTicks();
@@ -120,8 +154,9 @@ level::level(const std::string& level_cfg)
 				}
 			}
 
+			wml::const_node_ptr n = node;
 			if(node->has_attr("id")) {
-				node = load_level_wml(node->attr("id"));
+				n = load_level_wml(node->attr("id"));
 			} else {
 				//this save was made before we saved level ID's. The best
 				//we can do is get a level with a matching title.
@@ -137,23 +172,36 @@ level::level(const std::string& level_cfg)
 						if(lvl_info->attr("dimensions").str() == node->attr("dimensions").str()) {
 							//the dimensions match so we're sure this is
 							//the right one.
-							node = lvl_info;
+							n = lvl_info;
 							break;
 						}
 
 						//tenatively guess this is the level, but keep
 						//searching in case one with an exact dimensional
 						//match is found.
-						node = lvl_info;
+						n = lvl_info;
 					}
 				}
 			}
+			//in case the player had swallowed an object, copy it
+			//over from the savegame.
+			wml::node::const_child_iterator i1 = node->begin_child("serialized_objects");
+			wml::node::const_child_iterator i2 = n->begin_child("serialized_objects");
+			if(i1 != node->end_child("serialized_objects") && i2 != n->end_child("serialized_objects")) {
+				wml::node::const_child_iterator j1 = (i1->second)->begin_child("character");
+				wml::node::const_child_iterator j2 = (i1->second)->end_child("character");
+				while(j1 != j2) {
+					(i2->second)->add_child(j1->second);
+					++j1;
+				}
+			}
+			node = n;
 		}
 	}
 
+	dark_color_ = default_dark_color();
 	if(wml::get_bool(node, "dark", false)) {
 		dark_ = true;
-		dark_color_ = default_dark_color();
 	}
 
 	if(node->has_attr("dark_color")) {
@@ -169,11 +217,17 @@ level::level(const std::string& level_cfg)
 	}
 
 	segment_width_ = wml::get_int(node, "segment_width");
+	ASSERT_LOG(segment_width_%32 == 0, "segment_width in " << id_ << " is not divisible by 32");
+
+	segment_height_ = wml::get_int(node, "segment_height");
+	ASSERT_LOG(segment_height_%32 == 0, "segment_height in " << id_ << " is not divisible by 32");
 
 	music_ = node->attr("music");
 	replay_data_ = node->attr("replay_data");
 	cycle_ = wml::get_int(node, "cycle");
 	time_freeze_ = 0;
+	x_resolution_ = wml::get_int(node, "x_resolution");
+	y_resolution_ = wml::get_int(node, "y_resolution");
 	in_dialog_ = false;
 	title_ = node->attr("title");
 	if(node->has_attr("dimensions")) {
@@ -451,29 +505,33 @@ void level::load_character(wml::const_node_ptr c)
 void level::finish_loading()
 {
 	std::vector<sub_level_data> sub_levels;
-	if(segment_width_ > 0 && !editor_) {
-		int segment_number = 0;
-		for(int x = boundaries_.x(); x < boundaries_.x2(); x += segment_width_) {
-			level* sub_level = new level(*this);
-			const rect bounds(x, boundaries_.y(), segment_width_, boundaries_.h());
+	if((segment_width_ > 0 || segment_height_ > 0) && !editor_ && !preferences::compiling_tiles) {
 
-			sub_level->boundaries_ = bounds;
-			sub_level->tiles_.erase(std::remove_if(sub_level->tiles_.begin(), sub_level->tiles_.end(), boost::bind(level_tile_not_in_rect, bounds, _1)), sub_level->tiles_.end());
-			sub_level->solid_.clear();
-			sub_level->standable_.clear();
-			foreach(const level_tile& t, sub_level->tiles_) {
-				sub_level->add_tile_solid(t);
+		const int seg_width = segment_width_ > 0 ? segment_width_ : boundaries_.w();
+		const int seg_height = segment_height_ > 0 ? segment_height_ : boundaries_.h();
+
+		for(int y = boundaries_.y(); y < boundaries_.y2(); y += seg_height) {
+			for(int x = boundaries_.x(); x < boundaries_.x2(); x += seg_width) {
+				level* sub_level = new level(*this);
+				const rect bounds(x, y, seg_width, seg_height);
+
+				sub_level->boundaries_ = bounds;
+				sub_level->tiles_.erase(std::remove_if(sub_level->tiles_.begin(), sub_level->tiles_.end(), boost::bind(level_tile_not_in_rect, bounds, _1)), sub_level->tiles_.end());
+				sub_level->solid_.clear();
+				sub_level->standable_.clear();
+				foreach(const level_tile& t, sub_level->tiles_) {
+					sub_level->add_tile_solid(t);
+				}
+				sub_level->prepare_tiles_for_drawing();
+
+				sub_level_data data;
+				data.lvl.reset(sub_level);
+				data.xbase = x;
+				data.ybase = y;
+				data.xoffset = data.yoffset = 0;
+				data.active = false;
+				sub_levels.push_back(data);
 			}
-			sub_level->prepare_tiles_for_drawing();
-
-			sub_level_data data;
-			data.lvl.reset(sub_level);
-			data.xbase = x - boundaries_.x();
-			data.ybase = 0;
-			data.xoffset = data.yoffset = 0;
-			data.active = false;
-			sub_levels.push_back(data);
-			++segment_number;
 		}
 
 		const std::vector<entity_ptr> objects = get_chars();
@@ -502,6 +560,9 @@ void level::finish_loading()
 
 	graphics::texture::build_textures_from_worker_threads();
 
+	if (editor_ || preferences::compiling_tiles)
+		game_logic::set_verbatim_string_expressions (true);
+
 	game_logic::wml_formula_callable_read_scope read_scope;
 	foreach(wml::const_node_ptr node, wml_chars_) {
 		if(node->name() != "serialized_objects") {
@@ -544,6 +605,8 @@ void level::finish_loading()
 		}
 	}
 
+	game_logic::set_verbatim_string_expressions (false);
+
 	wml_chars_.clear();
 
 	controls::new_level(cycle_, players_.empty() ? 1 : players_.size(), multiplayer::slot());
@@ -558,17 +621,22 @@ void level::finish_loading()
 	}
 
 	if(!sub_levels.empty()) {
+		const int seg_width = segment_width_ > 0 ? segment_width_ : boundaries_.w();
+		const int seg_height = segment_height_ > 0 ? segment_height_ : boundaries_.h();
 		int segment_number = 0;
-		for(int x = boundaries_.x(); x < boundaries_.x2(); x += segment_width_) {
-			const std::vector<entity_ptr> objects = get_chars();
-			foreach(const entity_ptr& obj, objects) {
-				if(!obj->is_human() && obj->midpoint().x >= x && obj->midpoint().x < x + segment_width_ && obj->midpoint().y >= boundaries_.y() && obj->midpoint().y < boundaries_.y2()) {
-					sub_levels[segment_number].lvl->add_character(obj);
-					remove_character(obj);
+		for(int y = boundaries_.y(); y < boundaries_.y2(); y += seg_height) {
+			for(int x = boundaries_.x(); x < boundaries_.x2(); x += seg_width) {
+				const std::vector<entity_ptr> objects = get_chars();
+				foreach(const entity_ptr& obj, objects) {
+					if(!obj->is_human() && obj->midpoint().x >= x && obj->midpoint().x < x + seg_width && obj->midpoint().y >= y && obj->midpoint().y < y + seg_height) {
+						ASSERT_INDEX_INTO_VECTOR(segment_number, sub_levels);
+						sub_levels[segment_number].lvl->add_character(obj);
+						remove_character(obj);
+					}
 				}
-			}
 
-			++segment_number;
+				++segment_number;
+			}
 		}
 	}
 }
@@ -873,6 +941,13 @@ wml::node_ptr level::write() const
 	res->set_attr("title", title_);
 	res->set_attr("music", music_);
 	res->set_attr("segment_width", formatter() << segment_width_);
+	res->set_attr("segment_height", formatter() << segment_height_);
+
+	if(x_resolution_ || y_resolution_) {
+		res->set_attr("x_resolution", formatter() << x_resolution_);
+		res->set_attr("y_resolution", formatter() << y_resolution_);
+	}
+
 	if(gui_algo_str_ != "default") {
 		res->set_attr("gui", gui_algo_str_);
 	}
@@ -1452,11 +1527,11 @@ void level::prepare_tiles_for_drawing()
 	blit_cache_.clear();
 
 	for(int n = 0; n != tiles_.size(); ++n) {
-		if(tiles_[n].object->solid_color()) {
+		if(!is_arcade_level() && tiles_[n].object->solid_color()) {
 			continue;
 		}
 
-		if(tiles_[n].x <= boundaries().x() - TileSize || tiles_[n].y <= boundaries().y() - TileSize || tiles_[n].x >= boundaries().x2() || tiles_[n].y >= boundaries().y2()) {
+		if(!editor_ && (tiles_[n].x <= boundaries().x() - TileSize || tiles_[n].y <= boundaries().y() - TileSize || tiles_[n].x >= boundaries().x2() || tiles_[n].y >= boundaries().y2())) {
 			continue;
 		}
 
@@ -1477,11 +1552,11 @@ void level::prepare_tiles_for_drawing()
 	}
 
 	for(int n = 0; n != tiles_.size(); ++n) {
-		if(tiles_[n].x <= boundaries().x() - TileSize || tiles_[n].y <= boundaries().y() - TileSize || tiles_[n].x >= boundaries().x2() || tiles_[n].y >= boundaries().y2()) {
+		if(!editor_ && (tiles_[n].x <= boundaries().x() - TileSize || tiles_[n].y <= boundaries().y() - TileSize || tiles_[n].x >= boundaries().x2() || tiles_[n].y >= boundaries().y2())) {
 			continue;
 		}
 
-		if(tiles_[n].object->solid_color()) {
+		if(!is_arcade_level() && tiles_[n].object->solid_color()) {
 			tiles_[n].draw_disabled = true;
 			if(!solid_color_rects_.empty()) {
 				solid_color_rect& r = solid_color_rects_.back();
@@ -1568,9 +1643,9 @@ void level::prepare_tiles_for_drawing()
 namespace {
 bool sort_entity_drawing_pos(const entity_ptr& a, const entity_ptr& b) {
 	return a->zorder() < b->zorder() ||
-	       a->zorder() == b->zorder() && a->y() < b->y() ||
-		   a->zorder() == b->zorder() && a->y() < b->y() ||
-		   a->zorder() == b->zorder() && a->y() == b->y() && a->x() < b->x();
+		   a->zorder() == b->zorder() && a->zsub_order() < b->zsub_order() ||
+		   a->zorder() == b->zorder() && a->zsub_order() == b->zsub_order() && a->midpoint().y < b->midpoint().y ||
+		   a->zorder() == b->zorder() && a->zsub_order() == b->zsub_order() && a->midpoint().y == b->midpoint().y && a->midpoint().x < b->midpoint().x;
 }
 }
 
@@ -1586,6 +1661,32 @@ void level::draw_status() const
 	}
 }
 
+namespace {
+void draw_entity(const entity& obj, int x, int y, bool editor) {
+	const std::pair<int,int>* scroll_speed = obj.position_scale_millis();
+
+	if(scroll_speed) {
+		glPushMatrix();
+		const int scrollx = scroll_speed->first;
+		const int scrolly = scroll_speed->second;
+
+		const int diffx = ((scrollx - 1000)*x)/1000;
+		const int diffy = ((scrolly - 1000)*y)/1000;
+
+		glTranslatef(diffx, diffy, 0.0);
+	}
+
+	obj.draw();
+	if(editor) {
+		obj.draw_group();
+	}
+
+	if(scroll_speed) {
+		glPopMatrix();
+	}
+}
+}
+
 extern std::vector<rect> background_rects_drawn;
 
 void level::draw(int x, int y, int w, int h) const
@@ -1594,6 +1695,8 @@ void level::draw(int x, int y, int w, int h) const
 
 	const int start_x = x;
 	const int start_y = y;
+	const int start_w = w;
+	const int start_h = h;
 
 	const int ticks = SDL_GetTicks();
 	x -= widest_tile_;
@@ -1616,6 +1719,14 @@ void level::draw(int x, int y, int w, int h) const
 
 	std::vector<entity_ptr>::const_iterator entity_itor = chars.begin();
 
+	
+	/*std::cerr << "SUMMARY " << cycle_ << ": ";
+	foreach(const entity_ptr& e, chars_) {
+		std::cerr << e->debug_description() << "(" << e->zsub_order() << "):";
+	}
+	
+	std::cerr << "\n";*/
+	
 	bool water_drawn = true;
 	int water_zorder = 0;
 	if(water_) {
@@ -1625,52 +1736,6 @@ void level::draw(int x, int y, int w, int h) const
 
 	std::set<int>::const_iterator layer = layers_.begin();
 
-	int player_zorder = 0;
-	if(players_.empty() == false) {
-		player_zorder = players_.front()->zorder();
-	}
-
-	for(; layer != layers_.end() && *layer < player_zorder; ++layer) {
-
-		if(!water_drawn && *layer > water_zorder) {
-			water_->draw(x, y, w, h);
-			water_drawn = true;
-		}
-
-		while(entity_itor != chars.end() && (*entity_itor)->zorder() <= *layer) {
-			if(!(*entity_itor)->is_human()) {
-				const std::pair<int,int>* scroll_speed = (*entity_itor)->position_scale_millis();
-
-				if(scroll_speed) {
-					glPushMatrix();
-					const int scrollx = scroll_speed->first;
-					const int scrolly = scroll_speed->second;
-
-					const int diffx = ((scrollx - 1000)*x)/1000;
-					const int diffy = ((scrolly - 1000)*y)/1000;
-
-					glTranslatef(diffx, diffy, 0.0);
-				}
-
-				(*entity_itor)->draw();
-				if(editor_) {
-					(*entity_itor)->draw_group();
-				}
-
-				if(scroll_speed) {
-					glPopMatrix();
-				}
-			}
-			++entity_itor;
-		}
-
-		draw_layer(*layer, x, y, w, h);
-	}
-
-	foreach(const const_entity_ptr& p, players_) {
-		p->draw();
-	}
-
 	for(; layer != layers_.end(); ++layer) {
 		if(!water_drawn && *layer > water_zorder) {
 			water_->draw(x, y, w, h);
@@ -1678,12 +1743,7 @@ void level::draw(int x, int y, int w, int h) const
 		}
 
 		while(entity_itor != chars.end() && (*entity_itor)->zorder() <= *layer) {
-			if(!(*entity_itor)->is_human()) {
-				(*entity_itor)->draw();
-				if(editor_) {
-					(*entity_itor)->draw_group();
-				}
-			}
+			draw_entity(**entity_itor, x, y, editor_);
 			++entity_itor;
 		}
 
@@ -1696,13 +1756,7 @@ void level::draw(int x, int y, int w, int h) const
 	}
 
 	while(entity_itor != chars.end()) {
-		if(!(*entity_itor)->is_human()) {
-			(*entity_itor)->draw();
-			if(editor_) {
-
-				(*entity_itor)->draw_group();
-			}
-		}
+		draw_entity(**entity_itor, x, y, editor_);
 		++entity_itor;
 	}
 
@@ -1744,123 +1798,49 @@ void level::draw(int x, int y, int w, int h) const
 		background_->draw_foreground(start_x, start_y, 0.0, cycle());
 	}
 
-	if(dark_ && !editor_) {
-		const int strip_height = 2;
-		static std::vector<darkness_strip> strips, new_strips;
-		strips.clear();
+	calculate_lighting(start_x, start_y, start_w, start_h);
+}
 
-		const int x_threshhold = 64;
+void level::calculate_lighting(int x, int y, int w, int h) const
+{
+	if(!dark_ || editor_ || texture_frame_buffer::unsupported()) {
+		return;
+	}
 
-		for(int ypos = y; ypos < y + h; ypos += strip_height) {
-			new_strips.clear();
-			darkness_strip s;
-			s.alpha_left = s.alpha_right = 255;
-			s.area = rect(x - x_threshhold, ypos, w + x_threshhold*2, strip_height);
-			new_strips.push_back(s);
-			foreach(const entity_ptr& c, active_chars_) {
-				foreach(const light_ptr& lt, c->lights()) {
-					static std::vector<darkness_strip> swap_strips;
-					swap_strips.resize(4);
-					for(int n = 0; n != new_strips.size(); ++n) {
-						const int result = lt->split_strip(new_strips[n], &swap_strips[swap_strips.size()-4]);
-						if(result == -1) {
-							swap_strips[swap_strips.size()-4] = new_strips[n];
-							swap_strips.resize(swap_strips.size()+1);
-						} else {
-							swap_strips.resize(swap_strips.size()+result);
-						}
-					}
-
-					swap_strips.resize(swap_strips.size()-4);
-					swap_strips.swap(new_strips);
-				}
-			}
-
-			if(!strips.empty() && new_strips.size() == 1 && strips.back().area.x() == new_strips.back().area.x() && strips.back().area.x2() == new_strips.back().area.x2() && new_strips.back().alpha_left == 255 && new_strips.back().alpha_right == 255) {
-				strips.back().area = rect(strips.back().area.x(), strips.back().area.y(), strips.back().area.w(), strips.back().area.h() + new_strips.back().area.h());
-			} else {
-				foreach(const darkness_strip& s, new_strips) {
-					strips.push_back(s);
-				}
-			}
-		}
-
-		
-		if(strips.empty() == false) {
-			glDisable(GL_TEXTURE_2D);
-			glDisableClientState(GL_TEXTURE_COORD_ARRAY);
-
-			glColor4ub(dark_color_.r(), dark_color_.g(), dark_color_.b(), dark_color_.a());
-
-			static std::vector<GLshort> vertexes, alpha_vertexes;
-			static std::vector<unsigned char> colors;
-			vertexes.clear();
-			alpha_vertexes.clear();
-			colors.clear();
-			foreach(const darkness_strip& s, strips) {
-				if(s.alpha_left == 255 && s.alpha_right == 255) {
-					vertexes.push_back(s.area.x());
-					vertexes.push_back(s.area.y());
-					vertexes.push_back(s.area.x2());
-					vertexes.push_back(s.area.y());
-					vertexes.push_back(s.area.x());
-					vertexes.push_back(s.area.y2());
-					vertexes.push_back(s.area.x2());
-					vertexes.push_back(s.area.y());
-					vertexes.push_back(s.area.x2());
-					vertexes.push_back(s.area.y2());
-					vertexes.push_back(s.area.x());
-					vertexes.push_back(s.area.y2());
-				} else {
-					alpha_vertexes.push_back(s.area.x());
-					alpha_vertexes.push_back(s.area.y());
-					alpha_vertexes.push_back(s.area.x2());
-					alpha_vertexes.push_back(s.area.y());
-					alpha_vertexes.push_back(s.area.x());
-					alpha_vertexes.push_back(s.area.y2());
-					alpha_vertexes.push_back(s.area.x2());
-					alpha_vertexes.push_back(s.area.y());
-					alpha_vertexes.push_back(s.area.x2());
-					alpha_vertexes.push_back(s.area.y2());
-					alpha_vertexes.push_back(s.area.x());
-					alpha_vertexes.push_back(s.area.y2());
-
-#define ADD_COLOR(alpha) \
-	colors.push_back(dark_color_.r()); \
-	colors.push_back(dark_color_.g()); \
-	colors.push_back(dark_color_.b()); \
-	colors.push_back((alpha*dark_color_.a())/255);
-
-					ADD_COLOR(s.alpha_left);
-					ADD_COLOR(s.alpha_right);
-					ADD_COLOR(s.alpha_left);
-					ADD_COLOR(s.alpha_right);
-					ADD_COLOR(s.alpha_right);
-					ADD_COLOR(s.alpha_left);
-#undef ADD_COLOR
-				}
-			}
-
-			if(!vertexes.empty()) {
-				glVertexPointer(2, GL_SHORT, 0, &vertexes[0]);
-				glDrawArrays(GL_TRIANGLES, 0, vertexes.size()/2);
-			}
-
-			if(!alpha_vertexes.empty()) {
-				glShadeModel(GL_SMOOTH);
-				glEnableClientState(GL_COLOR_ARRAY);
-				glColorPointer(4, GL_UNSIGNED_BYTE, 0, &colors.front());
-				glVertexPointer(2, GL_SHORT, 0, &alpha_vertexes[0]);
-				glDrawArrays(GL_TRIANGLES, 0, alpha_vertexes.size()/2);
-				glDisableClientState(GL_COLOR_ARRAY);
-				glShadeModel(GL_FLAT);
-			}
-
-			glColor4f(1.0, 1.0, 1.0, 1.0);
-			glEnableClientState(GL_TEXTURE_COORD_ARRAY);
-			glEnable(GL_TEXTURE_2D);
+	//find all the lights in the level
+	static std::vector<const light*> lights;
+	lights.clear();
+	foreach(const entity_ptr& c, active_chars_) {
+		foreach(const light_ptr& lt, c->lights()) {
+			lights.push_back(lt.get());
 		}
 	}
+
+	{
+		glBlendFunc(GL_ONE, GL_ONE);
+		rect screen_area(x, y, w, h);
+		const texture_frame_buffer::render_scope scope;
+
+		glClearColor(dark_color_.r()/255.0, dark_color_.g()/255.0, dark_color_.b()/255.0, dark_color_.a()/255.0);
+		glClear(GL_COLOR_BUFFER_BIT);
+		const unsigned char color[] = { dark_color_.r(), dark_color_.g(), dark_color_.b(), dark_color_.a() };
+		foreach(const light* lt, lights) {
+			lt->draw(screen_area, color);
+		}
+	}
+
+	//now blit the light buffer onto the screen
+	texture_frame_buffer::set_as_current_texture();
+
+	const GLfloat tcarray[] = { 0, 0, 0, 1, 1, 0, 1, 1 };
+	const GLfloat tcarray_rotated[] = { 0, 1, 1, 1, 0, 0, 1, 0 };
+	GLfloat varray[] = { x, y + h, x, y, x + w, y + h, x + w, y };
+	glVertexPointer(2, GL_FLOAT, 0, varray);
+	glTexCoordPointer(2, GL_FLOAT, 0,
+	               preferences::screen_rotated() ? tcarray_rotated : tcarray);
+	glBlendFunc(GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA);
+	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 }
 
 void level::draw_debug_solid(int x, int y, int w, int h) const
@@ -1972,8 +1952,6 @@ void level::process()
 		gui_algorithm_->process(*this);
 	}
 
-	multiplayer::send_and_receive();
-
 	const int LevelPreloadFrequency = 500; //10 seconds
 	//see if we have levels to pre-load. Load one periodically.
 	if((cycle_%LevelPreloadFrequency) == 0) {
@@ -1985,6 +1963,8 @@ void level::process()
 
 	controls::read_local_controls();
 
+	multiplayer::send_and_receive();
+
 	do_processing();
 
 	if(speech_dialogs_.empty() == false) {
@@ -2041,6 +2021,7 @@ void level::do_processing()
 		} else { //char is inactive
 			if( c->dies_on_inactive() ){
 				if(c->label().empty() == false) {
+					c->die_with_no_event();
 					chars_by_label_.erase(c->label());
 				}
 				
@@ -2054,6 +2035,22 @@ void level::do_processing()
 	std::sort(active_chars_.begin(), active_chars_.end());
 	active_chars_.erase(std::unique(active_chars_.begin(), active_chars_.end()), active_chars_.end());
 	std::sort(active_chars_.begin(), active_chars_.end(), sort_entity_drawing_pos);
+	
+/*
+	std::cerr << "SUMMARY " << cycle_ << ": ";
+	foreach(const entity_ptr& e, chars_) {
+		std::cerr << e->debug_description() << "(" << (e->is_human() ? "HUMAN," : "") << e->centi_x() << "," << e->centi_y() << "):";
+	}
+
+	std::cerr << "\n";
+	*/
+
+	int checksum = 0;
+	foreach(const entity_ptr& e, chars_) {
+		checksum += e->x() + e->y();
+	}
+
+	controls::set_checksum(cycle_, checksum);
 
 	const int ActivationDistance = 700;
 
@@ -2236,6 +2233,24 @@ bool level::is_solid(const level_solid_map& map, int x, int y, int* friction, in
 	return false;
 }
 
+bool level::standable(const rect& r, int* friction, int* traction, int* damage) const
+{
+	const int ybegin = r.y();
+	const int yend = r.y2();
+	const int xbegin = r.x();
+	const int xend = r.x2();
+
+	for(int y = ybegin; y != yend; ++y) {
+		for(int x = xbegin; x != xend; ++x) {
+			if(standable(x, y, friction, traction, damage)) {
+				return true;
+			}
+		}
+	}
+
+	return false;
+}
+
 bool level::standable(int x, int y, int* friction, int* traction, int* damage) const
 {
 	if(is_solid(solid_, x, y, friction, traction, damage) ||
@@ -2612,12 +2627,12 @@ const level_tile* level::get_tile_at(int x, int y) const
 
 void level::remove_character(entity_ptr e)
 {
-	std::cerr << "removing char: '" << e->label() << "'\n";
+	//std::cerr << "removing char: '" << e->label() << "'\n";
 	if(e->label().empty() == false) {
 		chars_by_label_.erase(e->label());
 	}
 	chars_.erase(std::remove(chars_.begin(), chars_.end(), e), chars_.end());
-	std::cerr << "removed char: '" << e->label() << "'\n";
+	//std::cerr << "removed char: '" << e->label() << "'\n";
 }
 
 std::vector<entity_ptr> level::get_characters_in_rect(const rect& r) const
@@ -2632,17 +2647,42 @@ std::vector<entity_ptr> level::get_characters_in_rect(const rect& r) const
 	return res;
 }
 
-entity_ptr level::get_character_at_point(int x, int y) const
+std::vector<entity_ptr> level::get_characters_at_point(int x, int y) const
 {
+	std::vector<entity_ptr> result;
 	foreach(entity_ptr c, chars_) {
 		const int xpos = x - c->x();
 		const int ypos = y - c->y();
 		if(!c->is_alpha(x, y)) {
-			return c;
+			result.push_back(c);
 		}
 	}
 	
-	return entity_ptr();
+	return result;
+}
+
+entity_ptr level::get_next_character_at_point(int x, int y) const
+{
+	std::vector<entity_ptr> v = get_characters_at_point(x, y);
+	if(v.empty()) {
+		return entity_ptr();
+	}
+
+	if(editor_selection_.empty()) {
+		return v.front();
+	}
+
+	std::vector<entity_ptr>::iterator itor = std::find(v.begin(), v.end(), editor_selection_.back());
+	if(itor == v.end()) {
+		return v.front();
+	}
+
+	++itor;
+	if(itor == v.end()) {
+		itor = v.begin();
+	}
+
+	return *itor;
 }
 
 void level::add_solid_rect(int x1, int y1, int x2, int y2, int friction, int traction, int damage)
@@ -2722,6 +2762,18 @@ void level::set_solid(level_solid_map& map, int x, int y, int friction, int trac
 	}
 }
 
+void level::add_multi_player(entity_ptr p)
+{
+	last_touched_player_ = p;
+	p->get_player_info()->set_player_slot(players_.size());
+	players_.push_back(p);
+	chars_.push_back(p);
+	if(p->label().empty() == false) {
+		chars_by_label_[p->label()] = p;
+	}
+	layers_.insert(p->zorder());
+}
+
 void level::add_player(entity_ptr p)
 {
 	chars_.erase(std::remove(chars_.begin(), chars_.end(), player_), chars_.end());
@@ -2749,6 +2801,13 @@ void level::add_player(entity_ptr p)
 		}
 	}
 
+	const int difficulty = current_difficulty();
+	for(int n = 0; n != chars_.size(); ++n) {
+		if(chars_[n].get() != NULL && !chars_[n]->appears_at_difficulty(difficulty)) {
+			chars_[n] = entity_ptr();
+		}
+	}
+
 	chars_.erase(std::remove(chars_.begin(), chars_.end(), entity_ptr()), chars_.end());
 }
 
@@ -2874,15 +2933,15 @@ namespace {
 
 const std::string LevelProperties[] = {
   "cycle", "player", "in_dialog",
-  "local_player", "num_active", "active_chars", "chars",
-  "in_editor", "zoom", "focus", "gui", "id",
+  "local_player", "num_active", "active_chars", "chars", "players",
+  "in_editor", "zoom", "focus", "gui", "id", "dimensions",
 };
 
 enum LEVEL_PROPERTY_ID {
 	LEVEL_CYCLE, LEVEL_PLAYER, LEVEL_IN_DIALOG,
 	LEVEL_LOCAL_PLAYER, LEVEL_NUM_ACTIVE,
-	LEVEL_ACTIVE_CHARS, LEVEL_CHARS, LEVEL_IN_EDITOR, LEVEL_ZOOM,
-	LEVEL_FOCUS, LEVEL_GUI, LEVEL_ID
+	LEVEL_ACTIVE_CHARS, LEVEL_CHARS, LEVEL_PLAYERS, LEVEL_IN_EDITOR, LEVEL_ZOOM,
+	LEVEL_FOCUS, LEVEL_GUI, LEVEL_ID, LEVEL_DIMENSIONS,
 };
 }
 
@@ -2921,6 +2980,14 @@ variant level::get_value_by_slot(int slot) const
 
 		return variant(&v);
 	}
+	case LEVEL_PLAYERS: {
+		std::vector<variant> v;
+		foreach(const entity_ptr& e, players()) {
+			v.push_back(variant(e.get()));
+		}
+
+		return variant(&v);
+	}
 	case LEVEL_IN_EDITOR:
 		return variant(editor_);
 	case LEVEL_ZOOM:
@@ -2943,6 +3010,14 @@ variant level::get_value_by_slot(int slot) const
 	case LEVEL_ID: {
 		return variant(id_);
 	}
+	case LEVEL_DIMENSIONS: {
+		std::vector<variant> v;
+		v.push_back(variant(boundaries_.x()));
+		v.push_back(variant(boundaries_.y()));
+		v.push_back(variant(boundaries_.x2()));
+		v.push_back(variant(boundaries_.y2()));
+		return variant(&v);
+	}
 	}
 
 	ASSERT_LOG(false, "BAD SLOT IN GET_VALUE FROM LEVEL " << slot);
@@ -3012,8 +3087,18 @@ variant level::get_value(const std::string& key) const
 		return variant(&v);
 	} else if(key == "segment_width") {
 		return variant(segment_width_);
+	} else if(key == "segment_height") {
+		return variant(segment_height_);
 	} else if(key == "num_segments") {
 		return variant(sub_levels_.size());
+	} else if(key == "camera_position") {
+		std::vector<variant> pos;
+		pos.reserve(4);
+		pos.push_back(variant(last_draw_position().x/100));
+		pos.push_back(variant(last_draw_position().y/100));
+		pos.push_back(variant(graphics::screen_width()));
+		pos.push_back(variant(graphics::screen_height()));
+		return variant(&pos);
 	} else {
 		const_entity_ptr e = get_entity_by_label(key);
 		if(e) {
@@ -3201,6 +3286,7 @@ bool level::can_interact(const rect& body) const
 
 void level::replay_from_cycle(int ncycle)
 {
+	return;
 	const int cycles_ago = cycle_ - ncycle;
 	if(cycles_ago <= 0) {
 		return;
@@ -3221,6 +3307,8 @@ void level::replay_from_cycle(int ncycle)
 
 void level::backup()
 {
+	return;
+
 	std::map<entity_ptr, entity_ptr> entity_map;
 
 	std::cerr << "BACKUP " << cycle_ << ": ";
@@ -3228,6 +3316,7 @@ void level::backup()
 	snapshot->rng_seed = rng::get_seed();
 	snapshot->cycle = cycle_;
 	snapshot->chars.reserve(chars_.size());
+
 	foreach(const entity_ptr& e, chars_) {
 		std::cerr << e->debug_description() << "(" << (e->is_human() ? "HUMAN," : "") << e->centi_x() << "," << e->centi_y() << "):";
 		snapshot->chars.push_back(e->backup());
@@ -3351,10 +3440,13 @@ void level::add_sub_level(const std::string& lvl, int xoffset, int yoffset, bool
 	itor->second.xoffset = xoffset - itor->second.xbase;
 	itor->second.yoffset = yoffset - itor->second.ybase;
 
+	std::cerr << "ADDING SUB LEVEL: " << lvl << "(" << itor->second.lvl->boundaries() << ") " << itor->second.xbase << ", " << itor->second.ybase << " -> " << itor->second.xoffset << ", " << itor->second.yoffset << "\n";
+
 	itor->second.active = true;
 	level& sub = *itor->second.lvl;
 
 	if(add_objects) {
+		const int difficulty = current_difficulty();
 		foreach(entity_ptr e, sub.chars_) {
 			if(e->is_human()) {
 				continue;
@@ -3366,10 +3458,12 @@ void level::add_sub_level(const std::string& lvl, int xoffset, int yoffset, bool
 			}
 
 			relocate_object(c, c->x() + itor->second.xoffset, c->y() + itor->second.yoffset);
-			add_character(c);
-			c->handle_event(OBJECT_EVENT_START_LEVEL);
+			if(c->appears_at_difficulty(difficulty)) {
+				add_character(c);
+				c->handle_event(OBJECT_EVENT_START_LEVEL);
 
-			itor->second.objects.push_back(c);
+				itor->second.objects.push_back(c);
+			}
 		}
 	}
 
@@ -3433,7 +3527,7 @@ void level::adjust_level_offset(int xoffset, int yoffset)
 	for(std::map<std::string, sub_level_data>::iterator i = sub_levels_.begin();
 	    i != sub_levels_.end(); ++i) {
 		if(i->second.active) {
-			add_sub_level(i->first, i->second.xoffset + xoffset + i->second.xbase, i->second.yoffset + yoffset, false);
+			add_sub_level(i->first, i->second.xoffset + xoffset + i->second.xbase, i->second.yoffset + yoffset + i->second.ybase, false);
 		}
 	}
 
@@ -3448,8 +3542,8 @@ bool level::relocate_object(entity_ptr e, int new_x, int new_y)
 	const int orig_x = e->x();
 	const int orig_y = e->y();
 
-	const int delta_x = e->x() - orig_x;
-	const int delta_y = e->y() - orig_y;
+	const int delta_x = new_x - orig_x;
+	const int delta_y = new_y - orig_y;
 
 	e->set_pos(new_x, new_y);
 
@@ -3490,11 +3584,36 @@ bool level::relocate_object(entity_ptr e, int new_x, int new_y)
 	return true;
 }
 
+void level::record_zorders()
+{
+	foreach(const level_tile& t, tiles_) {
+		t.object->record_zorder(t.zorder);
+	}
+}
+
+int level::current_difficulty() const
+{
+	if(!last_touched_player_) {
+		return 0;
+	}
+
+	playable_custom_object* p = dynamic_cast<playable_custom_object*>(last_touched_player_.get());
+	if(!p) {
+		return 0;
+	}
+
+	return p->difficulty();
+}
+
 UTILITY(correct_solidity)
 {
 	std::vector<std::string> files;
 	sys::get_files_in_dir(preferences::level_path(), &files);
 	foreach(const std::string& file, files) {
+		if(file.size() <= 4 || std::string(file.end()-4, file.end()) != ".cfg") {
+			continue;
+		}
+
 		boost::intrusive_ptr<level> lvl(new level(file));
 		lvl->finish_loading();
 		lvl->set_as_current_level();
@@ -3520,9 +3639,10 @@ UTILITY(correct_solidity)
 
 UTILITY(compile_levels)
 {
-	std::cerr << "COMPILING LEVELS...\n";
 	preferences::compiling_tiles = true;
 
+	std::cerr << "COMPILING LEVELS...\n";
+
 	std::vector<std::string> files;
 	sys::get_files_in_dir(preferences::level_path(), &files);
 
@@ -3532,6 +3652,7 @@ UTILITY(compile_levels)
 		std::cerr << "LOADING LEVEL '" << file << "'\n";
 		boost::intrusive_ptr<level> lvl(new level(file));
 		lvl->finish_loading();
+		lvl->record_zorders();
 		std::string data;
 		wml::write(lvl->write(), data);
 		sys::write_file("data/compiled/level/" + file, data);
