@@ -33,13 +33,14 @@
 #include "stats.hpp"
 #include "surface_cache.hpp"
 #include "text_entry_widget.hpp"
+#include "utils.hpp"
 #include "wml_node.hpp"
 #include "wml_writer.hpp"
 #include "wml_utils.hpp"
 #include "IMG_savepng.h"
+#include "globals.h"
 
 namespace {
-const int FrameTimeInMillis = 20;
 int skipping_game = 0;
 
 int global_pause_time;
@@ -70,7 +71,7 @@ void transition_scene(level& lvl, screen_position& screen_pos, bool transition_o
 
 		SDL_GL_SwapBuffers();
 
-		const int target_end_time = start_time + (n+1)*FrameTimeInMillis;
+		const int target_end_time = start_time + (n+1)*preferences::frame_time_millis();
 		const int current_time = SDL_GetTicks();
 		const int skip_time = target_end_time - current_time;
 		if(skip_time > 0) {
@@ -105,28 +106,82 @@ void iris_scene(const level& lvl, screen_position& screen_pos, float amount) {
 	if(lvl.player() == NULL) {
 		return;
 	}
-	
+
 	const_entity_ptr player = &lvl.player()->get_entity();
-	std::map<const_entity_ptr, std::vector<light_ptr> > lights;
-	lights[player].push_back(light_ptr(new circle_light(*dynamic_cast<const custom_object*>(player.get()), (1.0-amount)*500)));
-
-	foreach(entity_ptr e, lvl.get_active_chars()) {
-		e->swap_lights(lights[e]);
-	}
-
-	const bool dark_value = const_cast<level&>(lvl).set_dark(true);
+	const point light_pos = player->midpoint();
 
 	if(amount >= 0.99) {
 		SDL_Rect rect = {0, 0, graphics::screen_width(), graphics::screen_height()};
 		graphics::draw_rect(rect, graphics::color_black());
 	} else {
 		draw_scene(lvl, screen_pos);
-	}
 
-	const_cast<level&>(lvl).set_dark(dark_value);
+		const int screen_x = screen_pos.x/100;
+		const int screen_y = screen_pos.y/100;
 
-	foreach(entity_ptr e, lvl.get_active_chars()) {
-		e->swap_lights(lights[e]);
+		float radius_scale = 1.0 - amount;
+		const int radius = radius_scale*radius_scale*500;
+		const int center_x = -screen_x + light_pos.x;
+		const int center_y = -screen_y + light_pos.y;
+		SDL_Rect center_rect = {center_x - radius, center_y - radius, radius*2, radius*2 };
+
+		if(center_rect.y > 0) {
+			SDL_Rect top_rect = {0, 0, graphics::screen_width(), center_rect.y};
+			graphics::draw_rect(top_rect, graphics::color_black());
+		}
+
+		const int bot_rect_height = graphics::screen_height() - (center_rect.y + center_rect.h);
+		if(bot_rect_height > 0) {
+			SDL_Rect bot_rect = {0, graphics::screen_height() - bot_rect_height, graphics::screen_width(), bot_rect_height};
+			graphics::draw_rect(bot_rect, graphics::color_black());
+		}
+
+		if(center_rect.x > 0) {
+			SDL_Rect left_rect = {0, 0, center_rect.x, graphics::screen_height()};
+			graphics::draw_rect(left_rect, graphics::color_black());
+		}
+
+		const int right_rect_width = graphics::screen_width() - (center_rect.x + center_rect.w);
+		if(right_rect_width > 0) {
+			SDL_Rect right_rect = {graphics::screen_width() - right_rect_width, 0, right_rect_width, graphics::screen_height()};
+			graphics::draw_rect(right_rect, graphics::color_black());
+		}
+
+		static std::vector<float> x_angles;
+		static std::vector<float> y_angles;
+
+		if(x_angles.empty()) {
+			for(float angle = 0; angle < 3.1459*2.0; angle += 0.2) {
+				x_angles.push_back(cos(angle));
+				y_angles.push_back(sin(angle));
+			}
+		}
+
+
+		std::vector<GLfloat> varray;
+		for(int n = 0; n != x_angles.size(); ++n) {
+			const float xpos1 = center_x + radius*x_angles[n];
+			const float ypos1 = center_y + radius*y_angles[n];
+			const float xpos2 = center_x + (center_rect.w + radius)*x_angles[n];
+			const float ypos2 = center_y + (center_rect.h + radius)*y_angles[n];
+			varray.push_back(xpos1);
+			varray.push_back(ypos1);
+			varray.push_back(xpos2);
+			varray.push_back(ypos2);
+		}
+
+		glColor4ub(0, 0, 0, 255);
+
+		glDisable(GL_TEXTURE_2D);
+		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+
+		glVertexPointer(2, GL_FLOAT, 0, &varray.front());
+		glDrawArrays(GL_TRIANGLE_STRIP, 0, varray.size()/2);
+
+		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
+		glEnable(GL_TEXTURE_2D);
+
+		glColor4ub(255, 255, 255, 255);
 	}
 }
 
@@ -249,20 +304,20 @@ bool level_runner::play_cycle()
 
 	const bool is_multiplayer = controls::num_players() > 1;
 
-	int desired_end_time = start_time_ + pause_time_ + global_pause_time + cycle*FrameTimeInMillis + FrameTimeInMillis;
+	int desired_end_time = start_time_ + pause_time_ + global_pause_time + cycle*preferences::frame_time_millis() + preferences::frame_time_millis();
 
 	if(!is_multiplayer) {
 		const int ticks = SDL_GetTicks();
 		if(desired_end_time < ticks) {
-			const int new_desired_end_time = ticks + FrameTimeInMillis;
+			const int new_desired_end_time = ticks + preferences::frame_time_millis();
 			pause_time_ += new_desired_end_time - desired_end_time;
 			desired_end_time = new_desired_end_time;
 		}
 	}
 
-	//record player movement every 500 cycles on average.
+	//record player movement every 50 cycles on average.
 #if !TARGET_OS_IPHONE
-	if(rand()%500 == 0 && lvl_->player()) {
+	if(rand()%50 == 0 && lvl_->player()) {
 		point p = lvl_->player()->get_entity().midpoint();
 
 		if(last_stats_point_level_ == lvl_->id()) {
@@ -282,6 +337,10 @@ bool level_runner::play_cycle()
 	if(die_at > 0 && cycle >= die_at + 30) {
 		die_at = -1;
 
+		foreach(entity_ptr e, lvl_->get_chars()) {
+			e->handle_event(OBJECT_EVENT_PLAYER_DEATH);
+		}
+
 		//record stats of the player's death
 		lvl_->player()->get_entity().record_stats_movement();
 		stats::record_event(lvl_->id(), stats::record_ptr(new stats::die_record(lvl_->player()->get_entity().midpoint())));
@@ -342,6 +401,9 @@ bool level_runner::play_cycle()
 			player_info* player = lvl_->player();
 			if(player) {
 				player->get_entity().set_pos(portal->dest);
+				if(!player->get_entity().no_move_to_standing()){
+					player->get_entity().move_to_standing(*lvl_);
+				}
 			}
 		} else {
 			//the portal is to another level
@@ -401,7 +463,9 @@ bool level_runner::play_cycle()
 			if(player && portal->saved_game == false) {
 				player->get_entity().set_pos(dest);
 				new_level->add_player(&player->get_entity());
-				player->get_entity().move_to_standing(*new_level);
+				if(!player->get_entity().no_move_to_standing()){
+					player->get_entity().move_to_standing(*new_level);
+				}
 				player->get_entity().handle_event("enter_level");
 			} else {
 				player = new_level->player();
@@ -485,12 +549,16 @@ bool level_runner::play_cycle()
 				continue;
 			}
 #if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
+			// make sure nothing happens while the app is supposed to be "inactive"
 			case SDL_WINDOWEVENT:
 				if (event.window.event == SDL_WINDOWEVENT_MINIMIZED)
 				{
-					paused = true;
-				} else if (event.window.event == SDL_WINDOWEVENT_RESTORED) {
-					paused = false;
+					SDL_Event e;
+					while (SDL_WaitEvent(&e))
+					{
+						if (e.type == SDL_WINDOWEVENT && e.window.event == SDL_WINDOWEVENT_RESTORED)
+							break;
+					}
 				}
 			break;
 #endif
@@ -516,6 +584,15 @@ bool level_runner::play_cycle()
 					editor::edit(lvl_->id().c_str(), last_draw_position().x/100, last_draw_position().y/100);
 					lvl_.reset(load_level(editor::last_edited_level().c_str()));
 					lvl_->set_as_current_level();
+					if(lvl_->player()) {
+						//we want to save the game after leaving the editor
+						//so the game is restored to here if we die, rather
+						//than going to some other level. We must run
+						//the level through a process cycle just to make
+						//sure everything is set properly for the player.
+						lvl_->process();
+						lvl_->player()->get_entity().save_game();
+					}
 					pause_time_ += SDL_GetTicks();
 					#endif
 				} else if(key == SDLK_s && (mod&KMOD_CTRL)) {
@@ -582,9 +659,9 @@ bool level_runner::play_cycle()
 
 	if(message_dialog::get()) {
 		message_dialog::get()->process();
-		pause_time_ += FrameTimeInMillis;
+		pause_time_ += preferences::frame_time_millis();
 	} else {
-		if (!paused) {
+		if (!paused && pause_stack == 0) {
 			const int start_process = SDL_GetTicks();
 
 			try {
@@ -595,7 +672,7 @@ bool level_runner::play_cycle()
 
 			next_process_ += (SDL_GetTicks() - start_process);
 		} else {
-			pause_time_ += FrameTimeInMillis;
+			pause_time_ += preferences::frame_time_millis();
 		}
 	}
 
@@ -616,7 +693,9 @@ bool level_runner::play_cycle()
 
 		performance_data perf = { current_fps_, current_cycles_, current_delay_, current_draw_, current_process_, current_flip_, cycle, current_events_, profiling_summary_ };
 #if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE || TARGET_OS_MAEMO
-		settings_dialog.draw(in_speech_dialog());
+		if( ! is_achievement_displayed() ){
+			settings_dialog.draw(in_speech_dialog());
+		}
 #endif
 		
 		if(!is_skipping_game() && preferences::show_fps()) {
@@ -674,7 +753,7 @@ bool level_runner::play_cycle()
 		}
 	}
 
-	if (!paused) ++cycle;
+	if (!paused && pause_stack == 0) ++cycle;
 
 #ifdef TARGET_OS_MAEMO
 	if (should_halt)
@@ -685,11 +764,16 @@ bool level_runner::play_cycle()
 			SDL_Delay(500);
 			osso::process();
 			while(SDL_PollEvent(&event)) {
-				if (event.type == SDL_ACTIVEEVENT &&
-				    event.active.state & SDL_APPINPUTFOCUS &&
-				    event.active.gain == 1) {
-					should_halt = false;
+				switch(event.type) {
+				case SDL_ACTIVEEVENT:
+					if (event.active.state & SDL_APPINPUTFOCUS &&
+					    event.active.gain == 1)
+						should_halt = false;
 					break;
+				case SDL_QUIT:
+					stats::record_event(lvl_->id(), stats::record_ptr(new stats::quit_record(lvl_->player()->get_entity().midpoint())));
+					should_halt = false;
+					quit_ = true;
 				}
 			}
 		} while (should_halt);
@@ -700,13 +784,8 @@ bool level_runner::play_cycle()
 #if TARGET_IPHONE_SIMULATOR || TARGET_OS_IPHONE
 	if (quit_)
 	{
-		wml::node_ptr node = lvl_->write();
-		if(sound::current_music().empty() == false) {
-			node->set_attr("music", sound::current_music());
-		}
-		
-		sys::write_file(preferences::auto_save_file_path(), wml::output(node));
-		sys::write_file(std::string(preferences::auto_save_file_path()) + ".stat", "1");
+		write_autosave();
+		preferences::save_preferences();
 	}
 #endif
 	
